/*
 * Copyright (c) 2009 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _RMP440STRUCT_H
#define _RMP440STRUCT_H

/* Main modes of the module */
typedef enum RMP440_MODE {
	RMP440_IDLE = 0,	/* ready motors on */
	RMP440_MOTORS_OFF,	/* error motors off */
	RMP440_MANUAL,		/* joystick mode */
	RMP440_TRACK,		/* normal tracking mode */
	RMP440_EMERGENCY,	/* emergency 'Pause' mode */
	RMP440_POWER_CORD	/* power cord connected - don't move */
} RMP440_MODE;


typedef struct RMP440_STATUS_STR {
	RMP440_MODE rs_mode;
	int operational_state;	/* from rmp440 MCU 3 */
	double propulsion_battery_level;
	double aux_battery_level;
	double pitch, roll, yaw_rate;
	double v, w;
} RMP440_STATUS_STR;

/* Kinematics of the robot */
typedef struct RMP440_KINEMATICS_STR {
	double leftWheelRadius;	
	double rightWheelRadius;
	double axisWidth;
} RMP440_KINEMATICS_STR;

/* Current dynamic parameters, from the nvram */
typedef struct RMP440_DYNAMIC_STR {
	double vMax;
	double wMax;
	double linAccelMax;
	double angAccelMax;
} RMP440_DYNAMIC_STR;

/* Coefficient proportionnel a la vitesse pour calculer l'erreur sur 
   le deplacement des roues */
typedef struct RMP440_VAR_PARAMS {
	double coeffLin;
	double coeffAng;
	double coeffLinAng;
} RMP440_VAR_PARAMS;

/* Track parameters */
typedef struct RMP440_TRACK_PARAMS {
	GENPOS_POSTER_NAME posterName;
	GENPOS_TRACK_MODE mode;
} RMP440_TRACK_PARAMS;

/*----------------------------------------------------------------------*/
/* 3D */

/* a GENPOS_CART_STATE-like struct in 3D */
/* Same thing with 4 more elements: z,zC,pitch,roll */
typedef struct RMP440_CART_3DSTATE {
	double xRef;                  /* position  */
	double yRef;                  /* control point */
	double zRef;
	double xRob;                 /* position */
	double yRob;                 /* robot center */
	double zRob;
	double theta;              /* robot orientation*/
	double pitch;              /* robot's pitch (site) */
	double roll;               /* robot's roll (gite) */
	double v;                  /* linear velocity */
	double vt;
	double w;                  /* angular velocity */
} RMP440_CART_3DSTATE;

typedef struct RMP440_CART_3DCONFIG {
	double x;
	double y;
	double z;
	double theta;
} RMP440_CART_3DCONFIG;

/*----------------------------------------------------------------------*/
/* Odometry Method */

typedef enum RMP440_ODOMETRY_MODE {
	RMP440_ODO_2D, /* Standard 2D odometry */
	RMP440_ODO_3D  /* 3D odometry using Attitude Estimation 
			  provided by IMU */
} RMP440_ODOMETRY_MODE;

#define	RMP440_MAX_STRLEN	64
#define RMP440_DEFAULT_ATTITUDE_POSTER "imuAttitudeEstimation"

typedef struct RMP440_ODOMETRY_METHOD {
	RMP440_ODOMETRY_MODE mode;
	int unused;
	char attitudePosterName[RMP440_MAX_STRLEN];
} RMP440_ODOMETRY_METHOD;

/*----------------------------------------------------------------------*/
/* Gyro */

#ifdef GYRO
#include <gyroLib/gyroTypes.h>
#endif

typedef enum RMP440_GYRO_MODE {
	RMP440_GYRO_OFF,
	RMP440_GYRO_ON_IF_MOTION,
	RMP440_GYRO_ON
} RMP440_GYRO_MODE;

typedef struct RMP440_GYRO_PARAMS {
	RMP440_GYRO_MODE mode;
	char port[32];
#ifdef GYRO
	GYRO_TYPE type;
#endif
	double latitude;
	double woffset;
} RMP440_GYRO_PARAMS;

typedef struct RMP440_GYRO {
	RMP440_GYRO_PARAMS params;
	int gyroOn;
	RMP440_GYRO_MODE currentMode;
	double gyroToRobotOffset;
	double gyroTheta;
	double robotTheta;
	double gyroOmega;
} RMP440_GYRO;


typedef struct RMP440_GYRO_ASSERV {
	int enabled; // is asserv enabled
	
	int first; // first call to this function ?
	int straight; // is in straight mode ?
	double straight_angle; // angle for straight mode
	double prev_reference;
	double prev_measure;
	double prev_command;
	double prev_t;
	double integral_s;
	double jump_t; // last jump time
} RMP440_GYRO_ASSERV;

typedef struct RMP440_MAX_ACCEL {
	double prev_vel_command;
	double prev_vel_command_t;
} RMP440_MAX_ACCEL;

#endif
